﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.Shell.Interop;
using EnvDTE;
using System.Xml;
using System.Diagnostics;
using Microsoft.VisualStudio.Shell;
using System.ComponentModel.Design;
using Microsoft.VisualStudio.Shell.Design;

namespace ModelForm
{
    public static class VsHelper
    {
        public static IVsHierarchy GetCurrentHierarchy(IServiceProvider provider)
        {
            DTE vs = (DTE)provider.GetService(typeof(DTE));
            if (vs == null) throw new InvalidOperationException("DTE not found.");

            return ToHierarchy(vs.SelectedItems.Item(1).ProjectItem.ContainingProject);
        }
        public static IVsHierarchy ToHierarchy(EnvDTE.Project project)
        {
            if (project == null) throw new ArgumentNullException("project");

            string projectGuid = null;

            // DTE does not expose the project GUID that exists at in the msbuild project file.
            // Cannot use MSBuild object model because it uses a static instance of the Engine, 
            // and using the Project will cause it to be unloaded from the engine when the 
            // GC collects the variable that we declare.
            using (XmlReader projectReader = XmlReader.Create(project.FileName))
            {
                projectReader.MoveToContent();
                object nodeName = projectReader.NameTable.Add("ProjectGuid");
                while (projectReader.Read())
                {
                    if (Object.Equals(projectReader.LocalName, nodeName))
                    {
                        projectGuid = projectReader.ReadInnerXml();
                        break;
                    }
                }
            }

            Debug.Assert(!String.IsNullOrEmpty(projectGuid));

            IServiceProvider serviceProvider = new ServiceProvider(project.DTE as
                Microsoft.VisualStudio.OLE.Interop.IServiceProvider);

            return VsShellUtilities.GetHierarchy(serviceProvider, new Guid(projectGuid));
        }
        public static IVsProject3 ToVsProject(EnvDTE.Project project)
        {
            if (project == null) throw new ArgumentNullException("project");

            IVsProject3 vsProject = ToHierarchy(project) as IVsProject3;

            if (vsProject == null)
            {
                throw new ArgumentException("Project is not a VS project.");
            }

            return vsProject;
        }
        public static EnvDTE.Project ToDteProject(IVsHierarchy hierarchy)
        {
            if (hierarchy == null) throw new ArgumentNullException("hierarchy");

            object prjObject = null;
            if (hierarchy.GetProperty(0xfffffffe, -2027, out prjObject) >= 0)
            {
                return (EnvDTE.Project)prjObject;
            }
            else
            {
                throw new ArgumentException("Hierarchy is not a project.");
            }
        }
        public static EnvDTE.Project ToDteProject(IVsProject project)
        {
            if (project == null) throw new ArgumentNullException("project");

            return ToDteProject(project as IVsHierarchy);
        }

        public static IServiceProvider GetServiceProvider(EnvDTE.Project project)
        {
            IServiceProvider serviceProvider = new ServiceProvider(project.DTE as
                Microsoft.VisualStudio.OLE.Interop.IServiceProvider);

            //DynamicTypeService typeService = (DynamicTypeService)
            //    serviceProvider.GetService(typeof(DynamicTypeService));

            return serviceProvider;
        }
        private static ITypeResolutionService GetResolutionService(EnvDTE.Project project)
        {
            DynamicTypeService typeService = GetService<DynamicTypeService>(project);
            Debug.Assert(typeService != null, "No dynamic type service registered.");

            IVsSolution sln = GetService<IVsSolution>(project);
            IVsHierarchy hier;
            sln.GetProjectOfUniqueName(project.UniqueName, out hier);

            Debug.Assert(hier != null, "No active hierarchy is selected.");

            return typeService.GetTypeResolutionService(hier);
        }
        public static T GetService<T>(EnvDTE.Project project)
        {
            return (T)GetServiceProvider(project).GetService(typeof(T));
        }
        public static IDictionary<string, Type> GetAvailableTypes(IServiceProvider provider, bool includeReferences)
        {
            DynamicTypeService typeService = (DynamicTypeService)provider.GetService(typeof(DynamicTypeService));
            Debug.Assert(typeService != null, "No dynamic type service registered.");

            IVsHierarchy hier = VsHelper.GetCurrentHierarchy(provider);
            Debug.Assert(hier != null, "No active hierarchy is selected.");

            ITypeDiscoveryService discovery = typeService.GetTypeDiscoveryService(hier);
            Project dteProject = VsHelper.ToDteProject(hier);

            IDictionary<string, Type> availableTypes = new Dictionary<string, Type>();
            foreach (Type type in discovery.GetTypes(typeof(object), includeReferences))
            {
                // We will never allow non-public types selection, as it's terrible practice.
                if (type.IsPublic)
                {
                    if (!availableTypes.ContainsKey(type.FullName))
                    {
                        availableTypes.Add(type.FullName, type);
                    }
                }
            }
            return availableTypes;
        }
        public static Type GetType(EnvDTE.Project project, string typeName)
        {
            var types = GetAvailableTypes(GetServiceProvider(project), true);
            if (types.ContainsKey(typeName))
            {
                return types[typeName];
            }
            return null;
        }
    }
}
